/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2007  Klaas van Gend, Christofer Wesseling
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include "umtsmon_global.h"
#include "Device.h"
#include "PPPConnection.h"
#include "Popup.h"

#include "qlabel.h"
#include "qlineedit.h"



/** this class contains the observer for the PPPConnection
 *  it is owned and contained by mainwindow
 */
class MainWindowPPPObserver : public PPPObserverInterface
{
public:
	/// constructor
	MainWindowPPPObserver(mainwindow* aMainWindowPtr) : theMainWindowPtr(aMainWindowPtr)  {;};	
	virtual void newPPPState(PPPDState aNewState)
		{
			DEBUG2("MainWindowObserver::newPPPState %d\n", aNewState);
			theMainWindowPtr->newPPPState(aNewState);
		};			
	
private:
	mainwindow* theMainWindowPtr;
};


void mainwindow::init()
{
	theDevicePtr = NULL;	
	myRefreshTimer = NULL;
	
	setCaption(APPNAME " " APPRELEASE);
	menu_help_about->setMenuText("About " APPNAME );
	menu_connection_exit->setMenuText("Exit " APPNAME );
	Toolbar->setCaption(APPNAME" Toolbar");
	
	theMainWindowPPPObserverPtr = new MainWindowPPPObserver(this);
	thePPPConnection.attach(theMainWindowPPPObserverPtr);

	// lets check the PIN-Status... so we can make the menu user friendly... :)
	SIMHandler mySIMHandler;
	if(!mySIMHandler.isPINProtectionEnabled())
	{
	            DEBUG5(" PIN protection on SIM is disabled \n");		    
		    //let the user see whats active and not...
		    menu_pin_enable->setOn( false );	
		    menu_pin_enable->setEnabled(true);
		    
		    menu_pin_disable->setOn( true );
		    menu_pin_disable->setEnabled(false); 
		    
		    menu_pin_change->setEnabled(false);
		 
	}
	else
	{
	        DEBUG5(" PIN protection on SIM is enabled \n");
		//let the user see whats active and not...
		menu_pin_enable->setOn( true );	
		menu_pin_enable->setEnabled(false);
		
		menu_pin_disable->setOn( false );
		menu_pin_disable->setEnabled(true); 
		
		menu_pin_change->setEnabled(true);	       

	}
}


void mainwindow::showAboutDialog()
{
	DEBUG5("About dialog clicked\n");	
	AboutDialogUmtsmon myDialog;
	myDialog.versionlabel->setText("<b>"APPNAME"</b> version <b>"APPRELEASE"</b>");
	myDialog.exec();
}

void mainwindow::showInfoDialog()
{
	DEBUG5("info dialog clicked\n");	
	InfoDialogUmtsmon myDialog;
	myDialog.exec();
    
}



void mainwindow::DisablePINProtection()
{
    SIMHandler mySIMHandler;
    	EnterPIN myPIN;
			myPIN.enableValidator();					
			myPIN.setSIMHandlerPtr(&mySIMHandler);							
			myPIN.exec(); // shows the dialog, blocking all other..
			if(myPIN.result() == QDialog::Rejected) return; 
    if(!mySIMHandler.setPINActive(false,myPIN.pinField->text(),this))
	    	Popup::Warning("Disable PIN protection not successfully. \n Try again, and be sure to use the correct current PIN!");
    else
    {
	 //switch the toggle state....
	
	 //let the user see whats active and not...
	menu_pin_enable->setOn( false );	
	menu_pin_enable->setEnabled(true);
	
	menu_pin_disable->setOn( true );
	menu_pin_disable->setEnabled(false); 
	
	menu_pin_change->setEnabled(false);
	Popup::Info("Disable PIN protection was successfully!");

    }
    
}


void mainwindow::EnablePINProtection()
{
    SIMHandler mySIMHandler;
        	EnterPIN myPIN;
			myPIN.enableValidator();					
			myPIN.setSIMHandlerPtr(&mySIMHandler);							 
			myPIN.exec(); // shows the dialog, blocking all other..
		if(myPIN.result() == QDialog::Rejected) return;
    if(!mySIMHandler.setPINActive(true,myPIN.pinField->text(),this))
		Popup::Warning("Enable PIN protection not successfully. \n Try again, and be sure to use the correct current PIN!");
	 else
    {
	 //let the user see whats active and not...
	menu_pin_enable->setOn( true );	
	menu_pin_enable->setEnabled(false);
	
	menu_pin_disable->setOn( false );
	menu_pin_disable->setEnabled(true); 
	
	menu_pin_change->setEnabled(true);
	Popup::Info("PIN protection was successfully!");

    }
}


void mainwindow::ChangePIN()
{
    DEBUG5("change PIN settings clicked\n");
    SIMHandler mySIMHandler;
          EnterPIN myCurrentPIN;
			myCurrentPIN.enableValidator();					
			myCurrentPIN.setSIMHandlerPtr(&mySIMHandler);							
			myCurrentPIN.errorMessage->setText("Please enter your current PIN."); 
			myCurrentPIN.exec(); // shows the dialog, blocking all other..	
		if(myCurrentPIN.result() == QDialog::Rejected) return;
			
		EnterPIN myNewPIN;
			myNewPIN.enableValidator();					
			myNewPIN.setSIMHandlerPtr(&mySIMHandler);							
			myNewPIN.errorMessage->setText("Please enter your NEW pin."); 
			myNewPIN.exec(); // shows the dialog, blocking all other..
		if(myNewPIN.result() == QDialog::Rejected) return;
		
		EnterPIN myNewPINsec;
			myNewPINsec.enableValidator();					
			myNewPINsec.setSIMHandlerPtr(&mySIMHandler);							
			myNewPINsec.errorMessage->setText("Please verify your NEW pin!"); 
			myNewPINsec.exec(); // shows the dialog, blocking all other..
		if(myNewPINsec.result() == QDialog::Rejected) return;

	if(!myNewPINsec.pinField->text().contains(myNewPIN.pinField->text() ) )
	{
		Popup::Warning("Change PIN not successfully. \n\n New-PIN and verify-PIN were not same, please try again!");
	}
	else if(! mySIMHandler.setNewPIN(myCurrentPIN.pinField->text(),myNewPIN.pinField->text(),this))  
	{
			Popup::Warning("Change PIN not successfully. \n Try again, and be sure to use the correct current PIN!");
	}
	else
	{
		Popup::Info("PIN changed successfully!");
	}
}


void mainwindow::PushButtonCreateSMS_clicked()
{
	DEBUG5("mainwindow::PushButtonCreateSMS_clicked()\n");	
	if (checkRegistered()==true)
	{
		smsdialog myDialog;
		myDialog.exec();
	}
}



void mainwindow::setDevicePtr( Device* aDevicePtr )
{
	theDevicePtr = aDevicePtr;

	emit checkBoxAutoUpdateStats_stateChanged(2);
	emit updateNetworkData();
	
	thePPPConnection.setDeviceInfo(theDevicePtr);
}


void mainwindow::updateNetworkData()
{
	ConnectionInfo* myCIPtr = NULL;
	if (theDevicePtr == NULL)
		return;

	else
		myCIPtr = theDevicePtr->getConnectionInfoPtr();
	
	// there's three situations here:
	// 1) minimized - display in title bar
	// 2) maximized, in network tab - update fields inside that tab
	// 3) maximized, not in network - do nothing
	
	myCIPtr->refresh();
	thePPPConnection.refreshPPPStats();
	
	// and update title bar or fields with data
	if (isMinimized())
	{
		QString myTemp;
		QString myRT = myCIPtr->getRadiotype();
		QString myTotal = APPNAME " " + myRT + " " + myTemp.setNum(myCIPtr->getSignalQuality()); 
		setCaption( myTotal );
	}
	else
	{
		setCaption(APPNAME " " APPRELEASE);
		theLCDDisplay->setStrength(myCIPtr->getSignalQuality(), false);
		
		theLCDDisplay->setOperatorName( myCIPtr->getOperatorName(), false);
		theLCDDisplay->setRadioType( myCIPtr->getRadiotype(), false );
		
		theLCDDisplay->setUpload(QString::number(thePPPConnection.getCounterBytesSent(),10));
		theLCDDisplay->setDownload(QString::number(thePPPConnection.getCounterBytesReceived(),10));
	}
}





void mainwindow::checkBoxAutoUpdateStats_stateChanged(int aNewState)
{
	DEBUG5("checkBoxAutoUpdateStats_stateChanged, new state = %d\n", aNewState);

	if (aNewState == 2)
	{
		if (myRefreshTimer == NULL)
		{
			// setup the refreshtimer for network statistics and signal quality
			myRefreshTimer = new QTimer(this, "refreshtimer");
			connect(myRefreshTimer, SIGNAL(timeout()), SLOT(updateNetworkData()));
			updateNetworkData();
			myRefreshTimer->start(3000, FALSE);
		}
		else
		{
			DEBUG1("refreshtimer not null?!?");
		}
	}
	else
	{
		myRefreshTimer->stop();
		delete myRefreshTimer;
		myRefreshTimer = NULL;
	}
}

// only called from class MainWindowObserver
void mainwindow::newPPPState(PPPObserverInterface::PPPDState aNewState)
{
	DEBUG3("mainwindow::newPPPState()\n");
	switch(aNewState)
	{
	case PPPObserverInterface::STOPPED:
		// there is no connection
		menu_connection_connect->setOn( false );	
		menu_connection_disconnect->setOn( true );
		// so the 'disconnect' buttons are disabled anyway
		menu_connection_disconnect->setEnabled( false );
		toolbar_button_disconnect->setEnabled( false );
		// can we start a new connection?
		if (theDevicePtr)
			if (!theDevicePtr->getPPPSerialPortName().isEmpty())
			{
				menu_connection_connect->setEnabled( true );
				toolbar_button_connect->setEnabled( true );
				break;
			}
		else
		{
			DEBUG1("No DevicePtr setup\n");
		}
		// if we are still here, we cannot
		menu_connection_connect->setEnabled( false );
		toolbar_button_connect->setEnabled( false );
		break;
	case PPPObserverInterface::STARTING:
	case PPPObserverInterface::NOT_OURS_RUNNING:
		// there *is* a connection
		menu_connection_connect->setOn( true );	
		menu_connection_disconnect->setOn( false );
		// so the 'connect' buttons are disabled anyway
		menu_connection_connect->setEnabled( false );
		toolbar_button_connect->setEnabled( false );
		// and the PPPD is not (yet) ours - so we shouldn't disable it
		menu_connection_disconnect->setEnabled( false );
		toolbar_button_disconnect->setEnabled( false );
		break;
		break;
	case PPPObserverInterface::RUNNING:
		// there *is* a connection
		menu_connection_connect->setOn( true );	
		menu_connection_disconnect->setOn( false );
		// so the 'connect' buttons are disabled anyway
		menu_connection_connect->setEnabled( false );
		toolbar_button_connect->setEnabled( false );
		// and the PPPD is ours - so we are allowed to disable it
		menu_connection_disconnect->setEnabled( true );
		toolbar_button_disconnect->setEnabled( true );
		break;
	}
}

bool mainwindow::DisconnectPPP()
{
	DEBUG4("umtsmonmain::DisconnectPPP\n");
	thePPPConnection.stopPPP();
	return true;
}

bool mainwindow::checkRegistered()
{
	// ask a ConnectionInfo if we are operator connected already
	ConnectionInfo* myCIPtr = theDevicePtr->getConnectionInfoPtr();
	assert(myCIPtr != NULL);
	myCIPtr->refresh();
	if (!myCIPtr->isDeviceRegistered())
	{
		Popup::Warning("You are not connected to a network yet", this);
		return false;
	}
	return true;
}


bool mainwindow::ConnectPPP()
{
	DEBUG3("umtsmonmain::ConnectPPP\n");
	assert(theDevicePtr != NULL);
	
	ProgressDialog myProgress(this, "Setting up PPP Connection");

	if (checkRegistered()==false)
		return false;

	// we are not - let's start the PPP!
	// startPPP no longer blocks - so we can do adequate progress reports
	if(thePPPConnection.startPPP(theProfile)==false)
		goto problems;
	DEBUG1("start PPP successful!!!\n");

	myProgress.setMaxTime(30);
	// connection will take time anyway, so better show progress from start
	myProgress.start(0);
	while (thePPPConnection.isPPPStarting())
	{
		if (myProgress.runOneSecond() == false)
		{
			// Apparently, the user pressed cancel?!
			goto cleanup;
		}
	}
	myProgress.finish();
	if (thePPPConnection.hasErrors())
		goto problems;
	
	return true;
	
//---------------------
problems:	
	DEBUG1("start PPP NOT successful :-(\n");
	Popup::Warning("starting PPP was not successful\n"
	       "please run from commandline with -v5 and check output");
	
cleanup:
	sleep(1);
	DisconnectPPP();
	return false;
}


void mainwindow::showProfileDialog()
{
	DEBUG5(" profile dialog button clicked\n");	
	profiledialog myDialog;
	myDialog.setDevicePtr(theDevicePtr);
	myDialog.loadData();
	myDialog.exec();
}



void mainwindow::showNetworkOperatorDialog()
{
	DEBUG5(" network operator button clicked\n");

	//  FIXME: Only allow switching of operator if no connection active
	networkoperator myDialog;
	myDialog.exec();
	if(myDialog.result() == QDialog::Accepted)
		emit updateNetworkData();
}
