/* profiledialog
 * Copyright (C) 2006  Christofer Wesseling
 * Copyright (C) 2007  Christofer Wesseling
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include "umtsmon_global.h"
#include "Popup.h"
#include "Device.h"

void profiledialog::init()
{
	theDevicePtr = NULL;
	

}

// load the Data into the Dialog:  
void profiledialog::loadData()
{
	
	
	// add the profiles to the gui 
	connection_profile->insertStringList(theProfile.getProfileList());	
	// select the default Profile in the combo
	int i=0;
	
	while(i < connection_profile->count())
	{
		if (connection_profile->text(i) == theProfile.getProfileName())
			connection_profile->setCurrentItem(i);			
		i++;
	}	

	// more than 1 profile? then enable combo
	if(connection_profile->count() > 1)
		connection_profile->setEnabled(true);

	// FIXME: This code probably has to be moved due to bug #1631291 as well
	assert(theDevicePtr != NULL);
	// enable the Radio preferences widget and fill it 
	// with data from the Device class
	if (theDevicePtr->hasRadioPreferences())
	{
		textLabelRadioTypes->setEnabled(true);
		comboBoxRadioTypes->setEnabled(true);
		comboBoxRadioTypes->insertStringList(theDevicePtr->getRadioPreferences());
		comboBoxRadioTypes->setCurrentItem(theDevicePtr->getDefaultRadioPreference());
	}
	// set the fields/checkboxen using the getters defaults from Profile...
	updateCurrentProfile();	
}

void profiledialog::setDevicePtr( Device* aDevicePtr )
{
	theDevicePtr = aDevicePtr;
}

void profiledialog::pushButtonSaveProfile_clicked()
{
	//lets send the settings to de Profile... :)
	theProfile.setApn(connection_apn->text());
	theProfile.setUsername(connection_username->text());
	theProfile.setPassword(connection_password->text());
	theProfile.setNoIpDefault(connection_routing_noipdefault->isChecked());
	theProfile.setIsDefaultRoute(connection_routing_defaultroute->isChecked());
	theProfile.setReplaceDefaultRoute(connection_routing_replaceDefaultRoute->isChecked());
	theProfile.setUsePeerDns(connection_routing_usepeerdns->isChecked());
	theProfile.setNoCompression(connection_noCompression->isChecked());
	theProfile.setNoAuth(connection_noAuth->isChecked());
	theProfile.setRadioType(comboBoxRadioTypes->currentItem());

	// save a profile = set as deafault
	theProfile.setAsDefault();
	theProfile.save();
	pushButtonSaveProfile->setEnabled(false);
	//if we've more than 1 profile, enable combo
	if(connection_profile->count() > 1)
		connection_profile->setEnabled(true);
}
 
void profiledialog::pushButtonNewProfile_clicked()
{
	// you really want a new one? ok.. so you get new one...
	// lets help the user to se what he should do now...
	// disable temponary unused widgets

	NewProfileDialog newPrfDialog(this); //"this" means, put it in front of "this"-dialog	
	newPrfDialog.exec();
	if(newPrfDialog.result() == QDialog::Rejected)
	{
		DEBUG4("NewProfile: User clicked cancel\n");
		return; 
	}

	connection_apn->setText( newPrfDialog.newProfileName->text() );

	theProfile = Profile(newPrfDialog.newProfileName->text()) ;

	// insert into combo
	connection_profile->insertItem(theProfile.getProfileName());
	//select the Current Profile in the combo
	connection_profile->setCurrentItem( connection_profile->count()-1 );
	
	//set the fields/chckboxen using the getters defaults from Profile...	
	updateCurrentProfile();
	// more than one profile? enable combo
	if(connection_profile->count() >1 )
		connection_profile->setEnabled(true);
}

void profiledialog::pushButtonDeleteCurrentProfile_clicked()
{	
	Profile myProfile(connection_profile->currentText()) ;
	myProfile.remove();
	DEBUG4(" PROFILE '%s' has been deleted...\n", connection_profile->currentText().ascii());

	//and remove it from the combobox
	int i=0;
	while(i < connection_profile->count() )
	{
		if (connection_profile->text(i) == connection_profile->currentText())
			connection_profile->removeItem (i);		
		i++;
	}
	// did we have one or no profile? disable combo...
	if (connection_profile->count() <= 1) 
		connection_profile->setEnabled(false);
	// load the current
	updateCurrentProfile();
}

void profiledialog::pushButtonLoginHlp_clicked()
{
	Popup::Info("<p align=\"center\"><b>APN info</b></p> <p align=\"left\">For internet-connection, most providers accept an empty APN-Name.<br>So you'll <b>not</b> have to fill in APN, username,password.</p> <p align=\"center\">If your provider requires an APN, you can find a list of those here:<br> <b>http://www.reqwireless.com/apns.html</b></p> <p align=\"left\">For Private APNs, please ask your provider for support!</p>",this);
}

void profiledialog::enableButtonSaveProfile()
{
	pushButtonSaveProfile->setEnabled(true);
	if(connection_profile->count() >1) 
		connection_profile->setEnabled(true);
}

void profiledialog::updateCurrentProfile()
{
		theProfile = Profile( connection_profile->currentText() ); 
	//set the fields, getting defaults from Profile...
	connection_username->setText( theProfile.getUsername() );
	connection_password->setText( theProfile.getPassword() );	
	connection_apn->setText( theProfile.getAPN() );	
	
	//get the default values from the new Profile...
	connection_routing_noipdefault->setChecked(theProfile.getNoIpDefault());
	connection_routing_defaultroute->setChecked(theProfile.getIsDefaultRoute());
	connection_noCompression->setChecked(theProfile.getNoCompression());
	connection_noAuth->setChecked(theProfile.getNoAuth());
	connection_routing_replaceDefaultRoute->setChecked(theProfile.getReplaceDefaultRoute());
	connection_routing_usepeerdns->setChecked(theProfile.getUsePeerDns());
	comboBoxRadioTypes->setCurrentItem(theProfile.getRadioType());    
	// if the user want to the profile as default, he das to save..
	pushButtonSaveProfile->setEnabled(true); 
}


void profiledialog::comboBoxRadioType_activated(int anEntry)
{
	// FIXME: this code should not exist here - it should be a setting, stored in the config
	// file and read and acted on by PPPConnection. See Bug #1631291
    DEBUG5("comboBoxRadioType_activated: currently selected: '%s' is %d - %d\n\n", comboBoxRadioTypes->currentText().ascii(),
                                                 comboBoxRadioTypes->currentItem(), anEntry);
    if (!theDevicePtr->setRadioPreferences(anEntry))
    {
        QMessageBox myBox( APPNAME,
           "Setting your Radio technology preference didn't succeed",
            QMessageBox::Warning,
            QMessageBox::Ok | QMessageBox::Default,
            QMessageBox::NoButton, QMessageBox::NoButton);
        myBox.exec();
    }
}


void profiledialog::pushButtonOK_clicked()
{
	accept();
}
