/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006, 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef NETWORKCHANGER_CLASS
#define NETWORKCHANGER_CLASS

#include "ConnectionInfo.h"

#include <list>
#include <qstring.h>
#include <qstringlist.h>

class Query;


/// created to find all networks and change over to a new one
class NetworkChanger
{
public:
	NetworkChanger(void) : theQueryPtr(NULL) {};

	virtual ~NetworkChanger(); 


public:
	/** sends the AT commands to start searching for networks.
	    @returns  true if successful in sending, false if an error occurred
	*/
	bool startSearching(void);

	/** returns true if the search is completed. 
	    If so, it will call parseAnswer()
        */
	bool isSearchCompleted(void);

	/** asks the card to switch over to the specified network
	 *  @param  theNetworkID  theNetworkID from a CurrentNetworkItem
	 *  @returns  false if the network is forbidden
	 */
	bool startSwitchToNetworkID(long aNetworkID);

	enum SwitchResults{ NOT_READY_YET, OK, IS_CURRENT, ERROR};
	/** returns true if the switching is completed. 
        */
	SwitchResults isSwitchCompleted(void);


	/************ list of available networks interface ***********/
	// FIXME: I'd prefer to make this part private 
	enum IconType{ CURRENT, AVAILABLE, FORBIDDEN};
	struct CurrentNetworkItem
	{
		IconType  theIconType;
		QString   theNetworkName;
		long	  theNetworkID;
	};
	typedef std::list<CurrentNetworkItem*> CNlist;
	CNlist theCNlist;


private:
	Query* theQueryPtr;

	/** parse the answer from a single line of an AT+COPS=? query
	 *  depending on the card that generated the output, it can contain multiple operators
	 *  the results will then be available through theCNlist
	 * 
	 *  if the operator already exists in theCNlist, its name string will 
	 *  be appended to contain the new radio type 
	 */
	void parseLine(const QString& anAnswer);
	
	/** parse the AT+COPS string part between braces - containing a single operator
	 *  if successful, result will be added to theCNlist
	 */ 
	void parseOperator(const QString& aFullTelcoString);

	/** find in theCNlist a matching networkID
	 *  @returns NULL if not found
	 */
	CurrentNetworkItem*  findNetworkID(long aNetworkID);
	
	/** if an instance of the ConnectionInfoInhibitor exists, 
	 * ConnectionInfo is prohibited from sending out AT commands. 
	 * That's convenient as extra AT commands during network selection
	 * might f*ck up the network selection.
	 * As this object will exist always if a NetworkChanger exists, we're
	 * done quickly :-)
	 */
	ConnectionInfoInhibitor myConnectionInfoInhibitor;
};


#endif // NETWORKCHANGER
