/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "LCDDisplay.h"

#include <qpainter.h>

// unfortunately, there's quite a lot of initialising here :-(
// we cache a lot to speed up drawing

LCDDisplay::LCDDisplay( QWidget* aParent, const char* aName, WFlags aFlags) 
	: QLabel(aParent, aName, aFlags), theSignalWidth(0), theRadioType("n/a"), theProfile("n/a"),
	the14Font("Helvetica", 14), the10Font("Helvetica", 10),
	theBlack(20,20,20), theShadow(0x98, 0x9c, 0x62)
{
	QLabel::setPixmap(QPixmap::fromMimeSource("LCDBackground.png"));
	theGaugePixmap      = QPixmap::fromMimeSource("LCDSignal.png");

	the14Font.setPixelSize(14);
	the10Font.setPixelSize(11);
	
	// and... let's draw the thing!
	this->update();
}



void LCDDisplay::setStrength(unsigned int aNewStrength, bool needsUpdate)
{
	const unsigned int SIGNAL_SATURATION=24;
	
	if (aNewStrength == 99)
		theSignalWidth = -1;
	else
	{
		if (aNewStrength > SIGNAL_SATURATION)
			aNewStrength = SIGNAL_SATURATION;
		theSignalWidth = (theGaugePixmap.width() * aNewStrength) / SIGNAL_SATURATION;
	}

	if (needsUpdate)
		this->update();
}


void LCDDisplay::drawContents ( QPainter * p )
{
	// draws the background
	QLabel::drawContents(p);

	// let's make sure the string fits in the screen,
	// compensate font size for the string length
	if (theOperatorName.length() > 12)
		p->setFont(the10Font);
	else
		p->setFont(the14Font);
	p->setPen(theShadow);
	p->drawText(235,26, theOperatorName);
	
	p->setPen(theBlack);
	p->drawText(234,25, theOperatorName);
	
	
	p->setPen(theShadow);
	p->setFont(the10Font);
	
	p->drawText(235,58, theRadioType);
	p->drawText(235,75, theProfile);
	
	
	p->drawText(70,57, theUpload);
	p->drawText(70,75, theDownload);
	
	p->setPen(theBlack);
	p->drawText(234,57, theRadioType);
	p->drawText(234,74, theProfile);
	
	p->drawText(69,56, theUpload);
	p->drawText(69,74, theDownload);
	
	if (theSignalWidth == -1)
	{
		p->drawText(52,24, "n/a");
	}
	else
		p->drawPixmap( 42,17, theGaugePixmap, 0,0, theSignalWidth, theGaugePixmap.height() );
} 
